/*
rls.cc

This C++ file defines member functions for the class RLS_MODULE
It was generated with rcsdesign
with template version 1.1

MODIFICATIONS:
Sun Dec 17 20:48:06 EST 2000	Created with RCS-Design tool.

*/

// Include Files
#include "rcs.hh" 	// Common RCS definitions
#include "nml_mod.hh" 	// NML_MODULE definitions
#include "rls.hh" 	// RLS_MODULE definitions
// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.


#include "rlsn.hh" 	// NML Commands and Status definitions for rls
// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


// Constructor
RLS_MODULE::RLS_MODULE(int _is_base_class)
{

	// Initialize the NML channels if this module is not being used as the base class for another module.
	if(!_is_base_class)
	{
		INITIALIZE_NML();
	}
	// Add additional code to initialize the module here.
        num_of_params = NUM_OF_PARAMS;
        rls_estm = new RLS_ESTM(num_of_params);
        if (rls_estm->rls_estm_err != RLS_ESTM_NO_ERROR)
          {
            logError("Error in creating RLS_ESTM_MODULE object!!!");
            status = RCS_ERROR; // let my supervisor know I have a problem.
          }
	
	if ((param_estimates = new double[num_of_params]) == NULL)
	  {
	    logError("Error in creating KALMAN_FILTER_MODULE object!!!");
	    status = RCS_ERROR; // let my supervisor know I have a problem.
	  }
	
	if ((regression_vector = new double[num_of_params]) == NULL)
	  {
	    logError("Error in creating KALMAN_FILTER_MODULE object!!!");
	    status = RCS_ERROR; // let my supervisor know I have a problem.
	  }

}

// Overloaded Virtual Functions

/*
INITIALIZE_NML

The INITIALIZE_NML function is ussually called only once from within the
constructor. It should not be called if a derived class will also call it.

*/
void RLS_MODULE::INITIALIZE_NML()
{
	// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	setErrorLogChannel(new NML(nmlErrorFormat, "errlog", "rls", "rls_module.nml"));
	setCmdChannel(new RCS_CMD_CHANNEL(rlsFormat, "rls_cmd", "rls", "rls_module.nml"));
	rls_status = new RLS_STATUS();
	setStatChannel(new RCS_STAT_CHANNEL(rlsFormat, "rls_sts", "rls", "rls_module.nml"), rls_status);
	// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


}

/*
PRE_PROCESS

The PRE_PROCESS function is called every cycle after the command and
subordinates status have been read but before DECISION_PROCESS is called.
It is intended to be used for tasks such as sensory processing that should
be performed every cycle regardless of the current command or state.
*/
void RLS_MODULE::PRE_PROCESS()
{
	// Pre-Processing Code
  // Read the current plant output
  plant_output = 0;

  // Read (or caculate) the current regression vector
  for (int ii = 0; ii < num_of_params; ii++)
    {
      regression_vector[ii] = 0;
    }


}

/*
DECISION_PROCESS

The DECISION_PROCESS function is called every cycle as long as there is a non-zero command.
It is expected to call a command function based on commandInData->type.
*/
void RLS_MODULE::DECISION_PROCESS()
{
	switch(commandInData->type)
	{
		// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	case RLS_INIT_TYPE:
		INIT((RLS_INIT *)commandInData);
		break;

	case RLS_HALT_TYPE:
		HALT((RLS_HALT *)commandInData);
		break;

	case RLS_ESTIMATE_TYPE:
		ESTIMATE((RLS_ESTIMATE *)commandInData);
		break;

	case RLS_SET_COVARIANCE_TYPE:
		SET_COVARIANCE((RLS_SET_COVARIANCE *)commandInData);
		break;

	case RLS_SET_LAMBDA_TYPE:
		SET_LAMBDA((RLS_SET_LAMBDA *)commandInData);
		break;

	case RLS_SET_PARAM_TYPE:
		SET_PARAM((RLS_SET_PARAM *)commandInData);
		break;
		// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


	default:
		logError("The command %d is not recognized.",commandInData->type);
		break;
	}
}

/*
POST_PROCESS

The POST_PROCESS function is called every cycle after DECISION_PROCESS is called
but before the status and the subordinates commands  have been written.
It is intended to be used for tasks such as output filters that should
be performed every cycle regardless of the current command or state.
*/
void RLS_MODULE::POST_PROCESS()
{
  // Post-Processing Code
  // Report the current parameter estimates
  for (int ii = 0; ii < num_of_params; ii++)
    {
      rls_status->param_estimates[ii] = param_estimates[ii];
    }
  
  // Report the output error due to the current parameter estimates 
  rls_status->output_error = plant_output - estimated_output;
  
}

// Command Functions

/*
INIT

Parameter(s):
RLS_INIT *cmd_in -- NML Message sent from superior.

Most Modules will have an INIT command.
The INIT function is expected to initialize any variables that may be
in an uninitialized or unknown state, send INIT commands to its subordinates,
wait for the subordinates to be DONE and then inform its superior that it is done.
The state tables should use the STATE_MATCH macro so the diagnostics tool can 
highlight the current line in the state table.
*/
void RLS_MODULE::INIT(RLS_INIT *cmd_in)
{
	// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


	if(STATE_MATCH(NEW_COMMAND))
	{
		// Send an INIT command to all subordinates.
		// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

		// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


		stateNext(S1);
		// Reinitialize variables here.

	}
	// Wait for all subordinates to report done.
	// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	else if(STATE_MATCH(S1,
		1))
	// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.

	{
		status = RCS_DONE;
		stateNext(S2);
	}
	else if(STATE_MATCH(S2))
	{
		// Idle State
	}
}

/*
HALT

Parameter(s):
RLS_HALT *cmd_in -- NML Message sent from superior.

Most Modules will have an HALT command.
The HALT function is expected to stop any motion or command execution,
send HALT commands to its subordinates,
wait for the subordinates to be DONE and then inform its superior that it is done.
The state tables should use the STATE_MATCH macro so the diagnostics tool can 
highlight the current line in the state table.
*/
void RLS_MODULE::HALT(RLS_HALT *cmd_in)
{
	// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


	if(STATE_MATCH(NEW_COMMAND))
	{
		//Send a HALT command to all subordinates.
		// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

		// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.

		stateNext(S1);
	}
	// Wait for all subordinates to report done.
	// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	else if(STATE_MATCH(S1,
		1))
	// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.

	{
		status = RCS_DONE;
		stateNext(S2);
	}
	else if(STATE_MATCH(S2))
	{
		// Idle State
	}
}

/*
ESTIMATE

Parameter(s):
RLS_ESTIMATE *cmd_in -- NML Message sent from superior.

*/
void RLS_MODULE::ESTIMATE(RLS_ESTIMATE *cmd_in)
{
  // Put state table for RLS_ESTIMATE here.
  if(STATE_MATCH(NEW_COMMAND))
    {
      status = RCS_EXEC;
      stateNext(S1);
    }
  else if(STATE_MATCH(S1))
    {
      estimated_output = rls_estm->estimate(plant_output, regression_vector, param_estimates);
    }
  else if(STATE_MATCH(S2))
    {
      // Idle State
    }
}

/*
SET_COVARIANCE

Parameter(s):
RLS_SET_COVARIANCE *cmd_in -- NML Message sent from superior.

*/
void RLS_MODULE::SET_COVARIANCE(RLS_SET_COVARIANCE *cmd_in)
{
  // Put state table for RLS_SET_COVARIANCE here.
  if(STATE_MATCH(NEW_COMMAND))
    {
      status = RCS_EXEC;
      stateNext(S1);
    }
  else if(STATE_MATCH(S1))
    {
      // Set the covariance matrix to diag_value*Identity
      rls_estm->set_P(cmd_in->diag_value);

      status = RCS_DONE;
      stateNext(S2);
    }
  else if(STATE_MATCH(S2))
    {
      // Idle State
    }
}

/*
  SET_LAMBDA
  
  Parameter(s):
  RLS_SET_LAMBDA *cmd_in -- NML Message sent from superior.
  
*/
void RLS_MODULE::SET_LAMBDA(RLS_SET_LAMBDA *cmd_in)
{
  // Put state table for RLS_SET_LAMBDA here.
  if(STATE_MATCH(NEW_COMMAND))
    {
      status = RCS_EXEC;
      stateNext(S1);
    }
  else if(STATE_MATCH(S1))
    {
      // Set the covariance matrix to diag_value*Identity
      rls_estm->set_lambda(cmd_in->lambda);

      status = RCS_DONE;
      stateNext(S2);
    }
  else if(STATE_MATCH(S2))
    {
      // Idle State
    }
}

/*
SET_PARAM

Parameter(s):
RLS_SET_PARAM *cmd_in -- NML Message sent from superior.

*/
void RLS_MODULE::SET_PARAM(RLS_SET_PARAM *cmd_in)
{
	// Put state table for RLS_SET_PARAM here.
  if(STATE_MATCH(NEW_COMMAND))
    {
      status = RCS_EXEC;
      stateNext(S1);
    }
  else if(STATE_MATCH(S1))
    {
      // Set the parameter vector to the specified value
      rls_estm->set_theta(cmd_in->params_to_set);

      status = RCS_DONE;
      stateNext(S2);
    }
  else if(STATE_MATCH(S2))
    {
      // Idle State
    }
}

